package com.aliyun.iot.aep.demo;

import com.aliyun.iot.aep.sdk.framework.sdk.ISDKDelegate;
import com.aliyun.iot.aep.sdk.framework.sdk.SDKConfigure;
import com.aliyun.iot.aep.sdk.threadpool.ThreadPool;
import com.aliyun.iot.aep.utils.ALog;

import org.json.JSONArray;
import org.json.JSONObject;

import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Set;

/**
 * Created by wuwang on 2017/10/30.
 */

public final class SDKManager {
    static final private String TAG = "SDKManager";

    static private final ArrayList<SDKConfigure> dependOnUiThreadSdkConfigures = new ArrayList<>();
    static private final ArrayList<SDKConfigure> unDependOnUiThreadSdkConfigures = new ArrayList<>();

    /* API */

    static public void prepareForInitSdk(final DemoApplication app) {
        _prepareConfigure(app);
    }

    static public void init_underUiThread(final DemoApplication app) {

        ThreadPool.MainThreadHandler.getInstance().post(new Runnable() {
            @Override
            public void run() {
                _init_sdks(app, true);
            }
        });

        //_init_sdks(app, true);
    }

    static public void init_outOfUiThread(final DemoApplication app) {
        ThreadPool.DefaultThreadPool.getInstance().submit(new Runnable() {
            @Override
            public void run() {
                _init_sdks(app, false);
            }
        });
    }

    static public void dumpConfigure() {
        ALog.d(TAG, "\ndumpConfigure(): --- S ---");

        for (SDKConfigure configure : dependOnUiThreadSdkConfigures) {
            ALog.d(TAG, "dependOnUiThreadSdkConfigures(): " + configure.name);
        }

        for (SDKConfigure configure : unDependOnUiThreadSdkConfigures) {
            ALog.d(TAG, "unDependOnUiThreadSdkConfigures(): " + configure.name);
        }

        ALog.d(TAG, "dumpConfigure(): --- E ---\n");
    }

    /* methods: init */

    static private void _init_sdks(final DemoApplication app, final boolean initUISdk) {
        ArrayList<SDKConfigure> configures = (initUISdk ? dependOnUiThreadSdkConfigures : unDependOnUiThreadSdkConfigures);
        if (null == configures || configures.isEmpty())
            return;

        ISDKDelegate delegate = null;
        for (SDKConfigure config : configures) {
            if (initUISdk != config.initializedInUIThread)
                continue;

            /* continue if the sdk is already initialized */
            if (InitResultHolder.isInitialized(config.initializerClass)) {
                ALog.d(TAG, "the sdk \'" + config.name + "\' is already initialized");
                continue;
            }

            /* prepare the sdk initializer */
            delegate = _prepareSdkDelegate(app, config);
            if (null == delegate)
                continue;

            int resultCode = Integer.MAX_VALUE;
            try {
                resultCode = delegate.init(app, config, EnvConfigure.envArgs);
                ALog.d(TAG, "the sdk \'" + config.name + "\' is initialized");
            } catch (Exception ex) {
                ALog.e(TAG, "sdk initializing is failed: " + config.name, ex);
            }

            Result result = new Result();
            result.bInitialized = true;
            result.resultCode = resultCode;
            result.sdkName = config.name;
            result.sdkVer = config.version;
            InitResultHolder.updateResult(config.initializerClass, result);
        }
    }

    /* methods: helper */

    static private String readFile(InputStream configureInputStream) {
        StringBuilder ret = null;

        BufferedReader reader = null;
        try {
            reader = new BufferedReader(new InputStreamReader(configureInputStream));

            StringBuilder stringBuilder = new StringBuilder();
            String line = null;
            while (null != (line = reader.readLine())) {
                stringBuilder.append(line).append("\n");
            }

            ret = stringBuilder;
        } catch (Exception ex) {
            ALog.e(TAG, "read-file", ex);
        }

        try {
            if (null != reader)
                reader.close();
        } catch (Exception ex) {
            ALog.e(TAG, "read-file", ex);
        }

        return null != ret ? ret.toString() : null;
    }

    static private void _prepareConfigure(final DemoApplication app) {
        if (null == app) {
            return;
        }

        try {
            InputStream configureInputStream = app.getAssets().open("sdk_config.json");

            String body = readFile(configureInputStream);
            JSONArray configureJSONArray = (null != body && 0 < body.length()) ? new JSONArray(body) : null;

            for (int i = (null != configureJSONArray ? configureJSONArray.length() - 1 : -1); i >= 0; --i) {
                JSONObject jsonObject = configureJSONArray.getJSONObject(i);
                if (null == jsonObject)
                    continue;

                String n = jsonObject.optString("name");
                String c = jsonObject.optString("initializerClass");
                String v = jsonObject.optString("version");
                boolean ui = jsonObject.optBoolean("initializedInUIThread");
                boolean iotToken = jsonObject.optBoolean("needIoTToken");
                JSONObject opts = jsonObject.optJSONObject("opts");

                if (ui)
                    dependOnUiThreadSdkConfigures.add(new SDKConfigure(n, v, c, ui, iotToken, opts));
                else
                    unDependOnUiThreadSdkConfigures.add(new SDKConfigure(n, v, c, ui, iotToken, opts));
            }
        } catch (Exception ex) {
            ALog.e(TAG, "prepare-configure", ex);
        }

    }

    static private ISDKDelegate _prepareSdkDelegate(DemoApplication app, SDKConfigure configure) {
        ISDKDelegate ret = null;
        if (null == app || null == configure || null == configure.initializerClass)
            return ret;

        try {
            Class clazz = Class.forName(configure.initializerClass);
            if (null != clazz && ISDKDelegate.class.isAssignableFrom(clazz)) {
                ret = (ISDKDelegate) clazz.newInstance();
            }
        } catch (Exception ex) {
            ALog.e(TAG, "prepare sdk failed: " + configure.name);
        }


        return ret;
    }

    /* Inner type */

    static public class InitResultHolder {
        static private final HashMap<String, Result> delegateMap = new HashMap<>();

        /* API */

        static public boolean isInitialized(String initializerClass) {
            Result result = contain(initializerClass)
                    ? delegateMap.get(initializerClass)
                    : null;

            return null != result && result.bInitialized;
        }

        static public int getInitResultCode(String initializerClass) {
            Result result = delegateMap.containsKey(initializerClass)
                    ? delegateMap.get(initializerClass)
                    : null;

            return null != result
                    ? result.resultCode
                    : Integer.MAX_VALUE;
        }

        static public boolean updateResult(String initializerClass, Result result) {
            if (null == initializerClass || 0 >= initializerClass.length() || null == result)
                return false;

            delegateMap.put(initializerClass, result);
            return true;
        }

        /* methods: helper */

        static private boolean contain(String initializerClass) {
            return delegateMap.containsKey(initializerClass);
        }

        static public void dump() {
            ALog.d(TAG, "\ndump: --- S ---");
            Set<String> keys = delegateMap.keySet();
            for (String key : keys) {
                ALog.d(TAG, "[" + key + "]: " + delegateMap.get(key).bInitialized + ", " + delegateMap.get(key).resultCode);
            }

            ALog.d(TAG, "dump: --- E --- \n");
        }

    }

    static public class Result {
        public boolean bInitialized = false;
        public int resultCode = Integer.MAX_VALUE;

        public String sdkName = null;
        public String sdkVer = null;
    }

}
